module control ( opcode, intr_signal, rs_index, rt_index, rd_index, imm_out, alu_func, shift_func, mult_func, branch_func, a_source_out, b_source_out, c_source_out, pc_source_out, mem_source_out, exception_out);
input [31:0] opcode;
input intr_signal;
output [5:0] rs_index;
output [5:0] rt_index;
output [5:0] rd_index;
output [15:0] imm_out;
output [3:0] alu_func;
output [1:0] shift_func;
output [3:0] mult_func;
output [2:0] branch_func;
output [1:0] a_source_out;
output [1:0] b_source_out;
output [2:0] c_source_out;
output [1:0] pc_source_out;
output [3:0] mem_source_out;
output exception_out;

reg [5:0] rs_index;
reg [5:0] rt_index;
reg [5:0] rd_index;
reg [15:0] imm_out;
reg [3:0] alu_func;
reg [1:0] shift_func;
reg [3:0] mult_func;
reg [2:0] branch_func;
reg [1:0] a_source_out;
reg [1:0] b_source_out;
reg [2:0] c_source_out;
reg [1:0] pc_source_out;
reg [3:0] mem_source_out;
reg exception_out;
//---
parameter MODE_MULT = 1'b1;
parameter MODE_DIV  = 1'b0;
parameter MULT_NOTHING       = 4'b0000;
parameter MULT_READ_LO       = 4'b0001;
parameter MULT_READ_HI       = 4'b0010;
parameter MULT_WRITE_LO      = 4'b0011;
parameter MULT_WRITE_HI      = 4'b0100;
parameter MULT_MULT          = 4'b0101;
parameter MULT_SIGNED_MULT   = 4'b0110;
parameter MULT_DIVIDE        = 4'b0111;
parameter MULT_SIGNED_DIVIDE = 4'b1000;
//--
parameter FROM_INC4       = 2'b00;
parameter FROM_OPCODE25_0 = 2'b01;
parameter FROM_BRANCH     = 2'b10;
parameter FROM_LBRANCH    = 2'b11;
//--
parameter MEM_FETCH   = 4'b0000;
parameter MEM_READ32  = 4'b0100;
parameter MEM_WRITE32 = 4'b0101;
parameter MEM_READ16  = 4'b1000;
parameter MEM_READ16S = 4'b1010;
parameter MEM_WRITE16 = 4'b1001;
parameter MEM_READ8   = 4'b1100;
parameter MEM_READ8S  = 4'b1110;
parameter MEM_WRITE8  = 4'b1101;
//--
parameter ALU_NOTHING   		= 4'b0000;
parameter ALU_ADD        		= 4'b0001;
parameter ALU_SUBTRACT    		= 4'b0010;
parameter ALU_LESS_THAN   		= 4'b0011;
parameter ALU_LESS_THAN_SIGNED  = 4'b0100;
parameter ALU_OR          		= 4'b0101;
parameter ALU_AND         		= 4'b0110;
parameter ALU_XOR         		= 4'b0111;
parameter ALU_NOR         		= 4'b1000;
//--
parameter SHIFT_NOTHING        = 2'b00;
parameter SHIFT_LEFT_UNSIGNED  = 2'b01;
parameter SHIFT_RIGHT_SIGNED   = 2'b11;
parameter SHIFT_RIGHT_UNSIGNED = 2'b10;
parameter ZERO          		= 32'b00000000000000000000000000000000;
//--
parameter A_FROM_REG_SOURCE 	=  2'b00;
parameter A_FROM_IMM10_6    	=  2'b01;
parameter A_FROM_PC         	=  2'b10;
parameter B_FROM_REG_TARGET 	= 2'b00;
parameter B_FROM_IMM        	= 2'b01;
parameter B_FROM_SIGNED_IMM 	= 2'b10;
parameter B_FROM_IMMX4      	= 2'b11;
parameter C_FROM_NULL       	= 3'b000;
parameter C_FROM_ALU        	= 3'b001;
parameter C_FROM_SHIFT      	= 3'b001;
parameter C_FROM_MULT       	= 3'b001;
parameter C_FROM_MEMORY     	= 3'b010;
parameter C_FROM_PC         	= 3'b011;
parameter C_FROM_PC_PLUS4   	= 3'b100;
parameter C_FROM_IMM_SHIFT16	= 3'b101;
parameter C_FROM_REG_SOURCEN	= 3'b110;
parameter C_FROM_PC_PAST    	= 3'b111;
parameter BRANCH_LTZ 			= 3'b000;
parameter BRANCH_LEZ 			= 3'b001;
parameter BRANCH_EQ  			= 3'b010;
parameter BRANCH_NE  			= 3'b011;
parameter BRANCH_GEZ 			= 3'b100;
parameter BRANCH_GTZ 			= 3'b101;
parameter BRANCH_YES 			= 3'b110;
parameter BRANCH_NO  			= 3'b111;
//---------------



reg [5:0] op;
reg [5:0] func;
reg is_syscall;  //
reg [5:0] rs;
reg [5:0] rt;
reg [5:0] rd;
reg [4:0] rtx;
reg [15:0] imm;
reg [3:0] alu_function;
reg [1:0] shift_function;
reg [3:0] mult_function;
reg [2:0] branch_function;
reg [1:0] a_source;
reg [1:0] b_source;
reg [2:0] c_source;
reg [1:0] pc_source;
reg [3:0] mem_source;

integer i = 0;

always @(opcode, intr_signal) 
begin
   alu_function = ALU_NOTHING;
   shift_function = SHIFT_NOTHING;
   mult_function = MULT_NOTHING;
   a_source = A_FROM_REG_SOURCE;
   b_source = B_FROM_REG_TARGET;
   c_source = C_FROM_NULL;
   pc_source = FROM_INC4;
   branch_function = BRANCH_EQ;
   mem_source = MEM_FETCH;
   op = opcode[31:26];
   rs = {1'b0 , opcode[25:21]};
   rt = {1'b0 , opcode[20:16]};
   rtx = opcode[20:16];
   rd = {1'b0 , opcode[15:11]};
   func = opcode[5:0];
   imm = opcode[15:0];
   is_syscall = 1'b0;
   
	case (op) 
	6'b000000:   //SPECIAL
		case (func)
		6'b000000:  // --SLL   r[rd]=r[rt]<<re;
		begin
			a_source = A_FROM_IMM10_6;
			c_source = C_FROM_SHIFT;
			shift_function = SHIFT_LEFT_UNSIGNED;
		end
		6'b000010:   //--SRL   r[rd]=u[rt]>>re;
		begin
			a_source = A_FROM_IMM10_6;
			c_source = C_FROM_SHIFT;
			shift_function = SHIFT_RIGHT_UNSIGNED;
		end
		6'b000011:  // --SRA   r[rd]=r[rt]>>re;
		begin
			a_source = A_FROM_IMM10_6;
			c_source = C_FROM_SHIFT;
			shift_function = SHIFT_RIGHT_SIGNED;
		end
		6'b000100 :  // --SLLV  r[rd]=r[rt]<<r[rs];
		begin
			c_source = C_FROM_SHIFT;
			shift_function = SHIFT_LEFT_UNSIGNED;
		end
		6'b000110 :   //--SRLV  r[rd]=u[rt]>>r[rs];
		begin
			c_source = C_FROM_SHIFT;
			shift_function = SHIFT_RIGHT_UNSIGNED;
		end
		6'b000111 :   //--SRAV  r[rd]=r[rt]>>r[rs];
		begin
			c_source = C_FROM_SHIFT;
			shift_function = SHIFT_RIGHT_SIGNED;
		end
		6'b001000 : //  --JR    s->pc_next=r[rs];
		begin
			pc_source = FROM_BRANCH;
			alu_function = ALU_ADD;
			branch_function = BRANCH_YES;
		end
		6'b001001: //   --JALR  r[rd]=s->pc_next; s->pc_next=r[rs];
		begin
			c_source = C_FROM_PC_PLUS4;
			pc_source = FROM_BRANCH;
			alu_function = ALU_ADD;
			branch_function = BRANCH_YES;
		end
//		6'b001010:   //--MOVZ  if(!r[rt]) r[rd]=r[rs]; /*IV*/
//--    c_source := C_FROM_REG_SOURCE_EQZ;
 //   begin
 //   end
//		6'b001011:  // --MOVN  if(r[rt]) r[rd]=r[rs];  /*IV*/
//--         c_source := FROM_REG_SOURCE_NEZ;
 // begin
 //   end
		6'b001100:  // --SYSCALL
//--         if(r[4]==0) printf("0x%8.8lx ",r[5]);
			is_syscall = 1'b1;
   
		6'b001101:  // --BREAK s->wakeup=1;
		begin
		is_syscall = 1'b1;
    end
//		6'b001111: 	//   --SYNC  s->wakeup=1;
//		begin
 //   end
		6'b010000: 	//   --MFHI  r[rd]=s->hi;
		begin
			c_source = C_FROM_MULT;
			mult_function = MULT_READ_HI;
		end
		6'b010001:  // --FTHI  s->hi=r[rs];
			mult_function = MULT_WRITE_HI;

		6'b010010: //  --MFLO  r[rd]=s->lo;
		begin
			c_source = C_FROM_MULT;
			mult_function = MULT_READ_LO;
		end
		6'b010011:  // --MTLO  s->lo=r[rs];
			mult_function = MULT_WRITE_LO;
			
		6'b011000:  // --MULT  s->lo=r[rs]*r[rt]; s->hi=0;
			mult_function = MULT_SIGNED_MULT;

		6'b011001:  // --MULTU s->lo=r[rs]*r[rt]; s->hi=0;
			mult_function = MULT_MULT;

		6'b011010: //  --DIV   s->lo=r[rs]/r[rt]; s->hi=r[rs]%r[rt];
			mult_function = MULT_SIGNED_DIVIDE;

		6'b011011:  // --DIVU  s->lo=r[rs]/r[rt]; s->hi=r[rs]%r[rt];
			mult_function = MULT_DIVIDE;

		6'b100000:  // --ADD   r[rd]=r[rs]+r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_ADD;
		end
		6'b100001:  // --ADDU  r[rd]=r[rs]+r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_ADD;
		end
		6'b100010:  // --SUB   r[rd]=r[rs]-r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_SUBTRACT;
		end
		6'b100011: //  --SUBU  r[rd]=r[rs]-r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_SUBTRACT;
		end
		6'b100100:  //   --AND   r[rd]=r[rs]&r[rt];
		begin	
			c_source = C_FROM_ALU;
			alu_function = ALU_AND;
		end
		6'b100101:  // --OR    r[rd]=r[rs]|r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_OR;
		end
		6'b100110:   //--XOR   r[rd]=r[rs]^r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_XOR;
		end
		6'b100111:  // --NOR   r[rd]=~(r[rs]|r[rt]);
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_NOR;
		end
		6'b101010:  //--SLT   r[rd]=r[rs]<r[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_LESS_THAN_SIGNED;
		end
		6'b101011:  // --SLTU  r[rd]=u[rs]<u[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_LESS_THAN;
		end
		6'b101101:  // --DADDU r[rd]=r[rs]+u[rt];
		begin
			c_source = C_FROM_ALU;
			alu_function = ALU_ADD;
		end
//		6'b110001: //  --TGEU
//		begin
//    end
//		6'b110010:  // --TLT
//	begin
//    end	
//		6'b110011:  // --TLTU
//		begin
//    end
//		6'b110100:  // --TEQ 
//		begin
//    end
//		6'b110110:  // --TNE
//		begin
//    end 
		default:
		begin
    end
		endcase

	6'b000001: //  --REGIMM
	begin
		rt = 6'b000000;
		rd = 6'b011111;
		a_source = A_FROM_PC;
		b_source = B_FROM_IMMX4;
		alu_function = ALU_ADD;
		pc_source = FROM_BRANCH;
		branch_function = BRANCH_GTZ;
		//--if(test) pc=pc+imm*4

		case (rtx)
		5'b10000:  // --BLTZAL  r[31]=s->pc_next; branch=r[rs]<0;
		begin
			c_source = C_FROM_PC_PLUS4;
			branch_function = BRANCH_LTZ;
		end
		5'b00000: //  --BLTZ    branch=r[rs]<0;
			branch_function = BRANCH_LTZ;
		5'b10001:  // --BGEZAL  r[31]=s->pc_next; branch=r[rs]>=0;
		begin
			c_source = C_FROM_PC_PLUS4;
			branch_function = BRANCH_GEZ;
		end
		5'b00001:  // --BGEZ    branch=r[rs]>=0;
			branch_function = BRANCH_GEZ;
		//5'b10010:  // --BLTZALL r[31]=s->pc_next; lbranch=r[rs]<0;
		//begin
		//	c_source = C_FROM_PC_PLUS4;
		//	pc_source = FROM_LBRANCH;
		//	branch_function = BRANCH_LTZ;
		//end
		//5'b00010: //--BLTZL   lbranch=r[rs]<0;
		//begin
		//	pc_source = FROM_LBRANCH;
		//	branch_function = BRANCH_LTZ;
		//end
		//5'b10011:  // --BGEZALL r[31]=s->pc_next; lbranch=r[rs]>=0;
        //begin
		//	c_source = C_FROM_PC_PLUS4;
		//	pc_source = FROM_LBRANCH;
		//	branch_function = BRANCH_GEZ;
		//end
		//5'b00011:  // --BGEZL   lbranch=r[rs]>=0;
		//begin
		//	pc_source = FROM_LBRANCH;
		//	branch_function = BRANCH_GEZ;
		//end
		default:
		begin
    end
		endcase
end
	6'b000011: //  --JAL    r[31]=s->pc_next; s->pc_next=(s->pc&0xf0000000)|target;
	begin
		c_source = C_FROM_PC_PLUS4;
		rd = 6'b011111;
		pc_source = FROM_OPCODE25_0;
	end
	6'b000010: // --J      s->pc_next=(s->pc&0xf0000000)|target; 
		pc_source = FROM_OPCODE25_0;

	6'b000100:  // --BEQ    branch=r[rs]==r[rt];
	begin
		a_source = A_FROM_PC;
		b_source = B_FROM_IMMX4;
		alu_function = ALU_ADD;
		pc_source = FROM_BRANCH;
		branch_function = BRANCH_EQ;
	end
	6'b000101:  // --BNE    branch=r[rs]!=r[rt];
	begin
		a_source = A_FROM_PC;
		b_source = B_FROM_IMMX4;
		alu_function = ALU_ADD;
		pc_source = FROM_BRANCH;
		branch_function = BRANCH_NE;
	end
	6'b000110:  // --BLEZ   branch=r[rs]<=0;
	begin
		a_source = A_FROM_PC;
		b_source = B_FROM_IMMX4;
		alu_function = ALU_ADD;
		pc_source = FROM_BRANCH;
		branch_function = BRANCH_LEZ;
	end
	6'b000111:  // --BGTZ   branch=r[rs]>0;
	begin
		a_source = A_FROM_PC;
		b_source = B_FROM_IMMX4;
		alu_function = ALU_ADD;
		pc_source = FROM_BRANCH;
		branch_function = BRANCH_GTZ;
	end
	6'b001000:  // --ADDI   r[rt]=r[rs]+(short)imm;
	begin
		b_source = B_FROM_SIGNED_IMM;
		c_source = C_FROM_ALU;
		rd = rt;
		alu_function = ALU_ADD;
	end
	6'b001001:  // --ADDIU  u[rt]=u[rs]+(short)imm;
	begin
		b_source = B_FROM_SIGNED_IMM;
		c_source = C_FROM_ALU;
		rd = rt;
		alu_function = ALU_ADD;
	end
	6'b001010:  // --SLTI   r[rt]=r[rs]<(short)imm;
    begin  
		b_source = B_FROM_SIGNED_IMM;
		c_source = C_FROM_ALU;
		rd = rt;
		alu_function = ALU_LESS_THAN_SIGNED;
	end
	6'b001011:  // --SLTIU  u[rt]=u[rs]<(unsigned long)(short)imm;
	begin
		b_source = B_FROM_IMM;
		c_source = C_FROM_ALU;
		rd = rt;
		alu_function = ALU_LESS_THAN;
	end
	6'b001100:  // --ANDI   r[rt]=r[rs]&imm;
	begin
		b_source = B_FROM_IMM;
		c_source = C_FROM_ALU;
		rd = rt;
		alu_function = ALU_AND;
	end
	6'b001101:  // --ORI    r[rt]=r[rs]|imm;
	begin
	  b_source = B_FROM_IMM;
      c_source = C_FROM_ALU;
      rd = rt;
      alu_function = ALU_OR;
	end
	6'b001110:  // --XORI   r[rt]=r[rs]^imm;
	begin	
		b_source = B_FROM_IMM;
		c_source = C_FROM_ALU;
		rd = rt;
		alu_function = ALU_XOR;
	end
	6'b001111:  // --LUI    r[rt]=(imm<<16);
	begin
      c_source = C_FROM_IMM_SHIFT16;
      rd = rt;
	end
	6'b010000: //  --COP0
	begin
		alu_function = ALU_OR;
		c_source = C_FROM_ALU;
		if ( opcode[23] == 1'b0) //  --move from CP0
        begin 
			rs = {1'b1, opcode[15:11]};
			rt = 6'b000000;
			rd = {1'b0, opcode[20:16]};
		end
		else                  //    --move to CP0
        begin 
			rs = 6'b000000;
			rd[5] = 1'b1;
			pc_source = FROM_BRANCH;   //--delay possible interrupt
			branch_function = BRANCH_NO;
		end
	end
	//6'b010001: //  --COP1
	//begin
    //end
	//6'b010010: //  --COP2
	//begin
    //end
	//6'b010011: //  --COP3
	//begin
    //end
	//6'b010100: //  --BEQL   lbranch=r[rs]==r[rt];
	//begin
	//	a_source = A_FROM_PC;
	//	b_source = B_FROM_IMMX4;
	//	alu_function = ALU_ADD;
	//	pc_source = FROM_LBRANCH;
	//	branch_function = BRANCH_EQ;
	//end
	//6'b010101:  // --BNEL   lbranch=r[rs]!=r[rt];
    //begin
	//	a_source = A_FROM_PC;
	//	b_source = B_FROM_IMMX4;
	//	alu_function = ALU_ADD;
	//	pc_source = FROM_LBRANCH;
	//	branch_function = BRANCH_NE;
	//end
	//6'b010110: // --BLEZL  lbranch=r[rs]<=0;
    //begin  
	//	a_source = A_FROM_PC;
	//	b_source = B_FROM_IMMX4;
	//	alu_function = ALU_ADD;
	//	pc_source = FROM_LBRANCH;
	//	branch_function = BRANCH_LEZ;
	//end
	//6'b010111:  //--BGTZL  lbranch=r[rs]>0;
	//begin
	//	a_source = A_FROM_PC;
	//	b_source = B_FROM_IMMX4;
	//	alu_function = ALU_ADD;
	//	pc_source = FROM_LBRANCH;
	//	branch_function = BRANCH_GTZ;
	//end
	6'b100000:  // --LB     r[rt]=*(signed char*)ptr;
	begin
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		rd = rt;
		c_source = C_FROM_MEMORY;
		mem_source = MEM_READ8S;   // --address=(short)imm+r[rs];
	end
	6'b100001:  // --LH     r[rt]=*(signed short*)ptr;
	begin
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		rd = rt;
		c_source = C_FROM_MEMORY;
		mem_source = MEM_READ16S;  // --address=(short)imm+r[rs];
	end
	6'b100010:  // --LWL    //Not Implemented
	begin
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		rd = rt;
		c_source = C_FROM_MEMORY;
		mem_source = MEM_READ32;
	end
	6'b100011: // --LW     r[rt]=*(long*)ptr;
    begin
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		rd = rt;
		c_source = C_FROM_MEMORY;
		mem_source = MEM_READ32;
	end
	6'b100100:  // --LBU    r[rt]=*(unsigned char*)ptr;
    begin  
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		rd = rt;
		c_source = C_FROM_MEMORY;
		mem_source = MEM_READ8;   // --address=(short)imm+r[rs];
	end
	6'b100101:   //--LHU    r[rt]=*(unsigned short*)ptr;
    begin  
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		rd = rt;
		c_source = C_FROM_MEMORY;
		mem_source = MEM_READ16;  //  --address=(short)imm+r[rs];
	end
	6'b100110:   //--LWR    //Not Implemented
	begin
		$display("fault detected\n") ;
	end
	6'b101000:   //--SB     *(char*)ptr=(char)r[rt];
    begin  
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		mem_source = MEM_WRITE8;   //--address=(short)imm+r[rs];
	end
	6'b101001:  // --SH     *(short*)ptr=(short)r[rt];
    begin  
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		mem_source = MEM_WRITE16;
	end
	6'b101010:  // --SWL    //Not Implemented
    begin  
	  a_source = A_FROM_REG_SOURCE;
      b_source = B_FROM_SIGNED_IMM;
      alu_function = ALU_ADD;
      mem_source = MEM_WRITE32;  //--address=(short)imm+r[rs];
	end
	6'b101011:  // --SW     *(long*)ptr=r[rt];
	begin
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_SIGNED_IMM;
		alu_function = ALU_ADD;
		mem_source = MEM_WRITE32; // --address=(short)imm+r[rs];
	end
	6'b101110:  // --SWR    //Not Implemented
	begin
		i = i + 1 ;
		$display("passd: i=> %d\n" , i) ;
	end
	//6'b101111:  // --CACHE
	//begin
	//  end
	//6'b110000:  // --LL     r[rt]=*(long*)ptr;
	//begin
	//  end
	//6'b110001:  // --LWC1
	//begin
	//  end 
	//6'b110010:  // --LWC2
	//begin
	//  end 
	//6'b110011:  // --LWC3
	//begin
	//  end 
	//6'b110101:  // --LDC1
	//begin
	//  end 
	//6'b110110:  // --LDC2 
	//begin
	//  end
	//6'b110111:  // --LDC3
	//begin
	//  end 
	//6'b111000:  // --SC     *(long*)ptr=r[rt]; r[rt]=1;
	//begin
	//  end
	//6'b111001:  // --SWC1
	//begin
	//  end 
	//6'b111010:  // --SWC2
	//begin
	//  end 
	//6'b111011:  // --SWC3
	//begin
	//  end 
	//6'b111101:  // --SDC1
	//begin
	//  end 
	//6'b111110:  // --SDC2
	//begin
	//  end 
	//6'b111111:  // --SDC3
	//begin
	//  end 
	default :
	begin
	  end
	endcase

	if (c_source == C_FROM_NULL )
		rd = 6'b000000;
	
	if (intr_signal == 1'b1 || is_syscall == 1'b1)
	begin
		rs = 6'b111111;
		rt = 6'b000000;
		rd = 6'b101110; // --save PC in EPC
		alu_function = ALU_OR;
		shift_function = SHIFT_NOTHING;
		mult_function = MULT_NOTHING;
		branch_function = BRANCH_YES;
		a_source = A_FROM_REG_SOURCE;
		b_source = B_FROM_REG_TARGET; 
		c_source = C_FROM_PC;		
		pc_source = FROM_LBRANCH;
		mem_source = MEM_FETCH;
		exception_out = 1'b1;
		end
	else
		exception_out = 1'b0;

	rs_index = rs;
	rt_index = rt;
	rd_index = rd;
	imm_out = imm;
	alu_func = alu_function;
	shift_func = shift_function;
	mult_func = mult_function;
	branch_func = branch_function;
	a_source_out = a_source;
	b_source_out = b_source;
	c_source_out = c_source;
	pc_source_out = pc_source;
	mem_source_out = mem_source;
end

endmodule
