module mem_ctrl 
		(clk         , 
        reset_in     ,
        pause_in     ,
        nullify_op   ,
        address_pc   ,
        opcode_out   ,

        address_in   ,
        mem_source   ,
        data_write   ,
        data_read    ,
        pause_out    ,
        
        address_next ,
        byte_we_next ,

        address      ,
        byte_we      ,
        data_w       ,
        data_r     );

input clk          ;// in std_logic;
input reset_in     ;// in std_logic;
input pause_in     ;// in std_logic;
input nullify_op   ;// in std_logic;
input [31:2] address_pc   ;// in std_logic_vector(31:2);
output [31:0] opcode_out   ;// out std_logic_vector(31:0);

input [31:0] address_in   ;// in std_logic_vector(31:0);
input [3:0] mem_source   ;// in mem_source_type;
input [31:0] data_write   ;// in std_logic_vector(31:0);
output  [31:0] data_read    ;// out std_logic_vector(31:0);
output  pause_out    ;// out std_logic;

output  [31:2] address_next ;// out std_logic_vector(31:2);
output  [3:0] byte_we_next ;// out std_logic_vector(3:0);

output  [31:2] address      ;// out std_logic_vector(31:2);
output  [3:0] byte_we      ;// out std_logic_vector(3:0);
output  [31:0] data_w       ;// out std_logic_vector(31:0);
input [31:0] data_r       ;// in std_logic_vector(31:0));
		
	parameter  MEM_FETCH   = 4'b0000;
	parameter  MEM_READ32  = 4'b0100;
	parameter  MEM_WRITE32 = 4'b0101;
	parameter  MEM_READ16  = 4'b1000;
	parameter  MEM_READ16S = 4'b1010;
	parameter  MEM_WRITE16 = 4'b1001;
	parameter  MEM_READ8   = 4'b1100;
	parameter  MEM_READ8S  = 4'b1110;
	parameter  MEM_WRITE8  = 4'b1101;  

//"00" = big_endian; "11" = little_endian
	parameter ENDIAN_MODE   = 2'b00;
	parameter STATE_ADDR   = 1'b0;
	parameter STATE_ACCESS = 1'b1;

	parameter  ZERO          = 32'b00000000000000000000000000000000;
	parameter  ONES          = 32'b11111111111111111111111111111111;  

reg [31:0] opcode_reg     ;//: std_logic_vector(31:0);
reg [31:0] next_opcode_reg;//: std_logic_vector(31:0);
reg [31:2] address_reg    ;//: std_logic_vector(31:2);
reg [3:0] byte_we_reg     ;//: std_logic_vector(3:0);
reg mem_state_reg;

reg [31:2] address_var    ;// std_logic_vector(31:2);
reg [31:0] data_read_var  ;// std_logic_vector(31:0);
reg [31:0] data_write_var ;// std_logic_vector(31:0);
reg [31:0] opcode_next    ;// std_logic_vector(31:0);
reg [3:0] byte_we_var     ;// std_logic_vector(3:0);
reg mem_state_next 		  ;// std_logic;
reg pause_var      		  ;// std_logic;
reg [1:0] bits            ;// std_logic_vector[1:0];	

 
always @(posedge clk)
begin
	if (reset_in == 1'b1 )
	begin
		mem_state_reg <= STATE_ADDR;
		opcode_reg <= ZERO;
		next_opcode_reg <= ZERO;
		address_reg <= ZERO[31:2];
		byte_we_reg <= 4'b0000;
	
	end
	else
	begin
		if (pause_in == 1'b0)
		begin
			address_reg <= address_var;
			byte_we_reg <= byte_we_var;
			mem_state_reg <= mem_state_next;
			opcode_reg <= opcode_next;
			if (mem_state_reg == STATE_ADDR)
			begin
				next_opcode_reg <= data_r;
			end
		end
	end
end

always @(pause_in, nullify_op, 
                  address_pc, address_in, mem_source, data_write, 
                  data_r, opcode_reg, next_opcode_reg, mem_state_reg)
				  //, address_reg, byte_we_reg)
begin
	byte_we_var = 4'b0000;
	pause_var = 1'b0;
	data_read_var = ZERO;
	data_write_var = ZERO;
	mem_state_next = mem_state_reg;
	opcode_next = opcode_reg;
	
	case (mem_source)
		MEM_READ32 :
		begin
			data_read_var = data_r;
		end

		MEM_READ16:
		begin
			if (address_in[1] == ENDIAN_MODE[1])
			begin
				data_read_var[15:0] = data_r[31:16];
			end
			else
			begin
				data_read_var[15:0] = data_r[15:0];
			end

			if (mem_source == MEM_READ16 || data_read_var[15] == 1'b0)
			begin
				data_read_var[31:16] = ZERO[31:16];
			end
			else
			begin
				data_read_var[31:16] = ONES[31:16];
			end		
		end
		MEM_READ16S :
		begin
			if (address_in[1] == ENDIAN_MODE[1])
			begin
				data_read_var[15:0] = data_r[31:16];
			end
			else
			begin
				data_read_var[15:0] = data_r[15:0];
			end

			if (mem_source == MEM_READ16 || data_read_var[15] == 1'b0)
			begin
				data_read_var[31:16] = ZERO[31:16];
			end
			else
			begin
				data_read_var[31:16] = ONES[31:16];
			end
		end
		
		MEM_READ8:
		begin		
			bits = address_in[1:0] ^ ENDIAN_MODE;
			case (bits)
				2'b00 : data_read_var[7:0] = data_r[31:24];
				2'b01 : data_read_var[7:0] = data_r[23:16];
				2'b10 : data_read_var[7:0] = data_r[15:8];
				default : data_read_var[7:0] = data_r[7:0];
			endcase
			if (mem_source == MEM_READ8 || data_read_var[7] == 1'b0)
			begin
				data_read_var[31:8] = ZERO[31:8];
			end
			else
			begin
				data_read_var[31:8] = ONES[31:8];
			end		
		end
		MEM_READ8S :
		begin
			bits = address_in[1:0] ^ ENDIAN_MODE;
			case (bits)
				2'b00 : data_read_var[7:0] = data_r[31:24];
				2'b01 : data_read_var[7:0] = data_r[23:16];
				2'b10 : data_read_var[7:0] = data_r[15:8];
				default : data_read_var[7:0] = data_r[7:0];
			endcase
			if (mem_source == MEM_READ8 || data_read_var[7] == 1'b0)
			begin
				data_read_var[31:8] = ZERO[31:8];
			end
			else
			begin
				data_read_var[31:8] = ONES[31:8];
			end
		end
		MEM_WRITE32 :
		begin
			data_write_var = data_write;
			byte_we_var = 4'b1111;
		end
		MEM_WRITE16 :
		begin
			data_write_var = {data_write[15:0] , data_write[15:0]};
			if (address_in[1] == ENDIAN_MODE[1])
			begin
				byte_we_var = 4'b1100;
			end
			else
			begin
				byte_we_var = 4'b0011;
			end
		end
	MEM_WRITE8 :
		begin
			data_write_var = {data_write[7:0] , data_write[7:0] , data_write[7:0] , data_write[7:0]};
			bits = address_in[1:0] ^ ENDIAN_MODE;
			case (bits)
				2'b00 :
				byte_we_var = 4'b1000; 
				2'b01 : 
				byte_we_var = 4'b0100; 
				2'b10 :
				byte_we_var = 4'b0010; 
				default :
				byte_we_var = 4'b0001; 
			endcase
		end
		//default :

	endcase

	if (mem_source == MEM_FETCH) //opcode fetch
	begin
		address_var = address_pc;
		opcode_next = data_r;
		mem_state_next = STATE_ADDR;
	end
	else
	begin
		if (mem_state_reg == STATE_ADDR) 
		begin
			if (pause_in == 1'b0)
			begin
				address_var = address_in[31:2];
				mem_state_next = STATE_ACCESS;
				pause_var = 1'b1;
			end
			else
			begin
				address_var = address_pc;
				byte_we_var = 4'b0000;
			end
		end
		else  //STATE_ACCESS
		begin
			if (pause_in == 1'b0)
			begin
				address_var = address_pc;
				opcode_next = next_opcode_reg;
				mem_state_next = STATE_ADDR;
				byte_we_var = 4'b0000;
			end
			else
			begin
				address_var = address_in[31:2];
				byte_we_var = 4'b0000;
			end
		end
	end
      
    if (nullify_op == 1'b1 && pause_in == 1'b0)
	begin
		opcode_next = ZERO;  //NOP after beql
	end  
        
end


assign   opcode_out = opcode_reg;
assign   data_read = data_read_var;
assign   pause_out = pause_var;

assign   address_next = address_var;
assign   byte_we_next = byte_we_var;

assign   address = address_reg;
assign   byte_we = byte_we_reg;
assign   data_w = data_write_var;


endmodule
