module mlite_cpu(clk, reset_in, intr_in, address_next, byte_we_next, address, byte_we, data_w, data_r, mem_pause);
input clk;
input reset_in;
input intr_in;
output [31:2]address_next;
output [3:0]byte_we_next;
output [31:2]address;
output [3:0] byte_we;
output[31:0] data_w;
input[31:0] data_r;
input mem_pause;   

parameter FROM_LBRANCH   = 2'b11;
parameter FROM_INC4   = 2'b00;
parameter pipeline_stages = 2;

   //--When using a two stage pipeline "sigD <= sig".
   //--When using a three stage pipeline "sigD <= sig when rising_edge(clk)",
  // --  so sigD is delayed by one clock cycle.
   wire [31:0]opcode ;
   wire [5:0]rs_index    ;
   wire [5:0]rt_index    ;
   wire [5:0]rd_index    ;
   wire [5:0] rd_indexD;
   wire [31:0]reg_source   ;
   wire [31:0]reg_target   ;
   wire [31:0]reg_dest     ;
   wire [31:0] reg_destD;
   wire [31:0]a_bus        ;
   wire [31:0] a_busD;  
   wire [31:0]b_bus        ;
   wire [31:0] b_busD      ;
   wire [31:0] c_bus;
   wire [31:0]c_alu        ;
   wire [31:0]c_shift      ;
   wire [31:0]c_mult       ;
   wire [31:0]c_memory     ;
   wire [15:0]imm        ;
   wire [31:2]pc_future  ;
   wire [31:2]pc_current ;
   wire [31:2]pc_plus4   ;
   wire [3:0] alu_func      ;
   wire [3:0] alu_funcD ;
   wire [1:0] shift_func    ;
   wire [1:0] shift_funcD;
   wire [3:0] mult_func      ;
   wire [3:0] mult_funcD;
   wire [2:0] branch_func    ;
   wire take_branch    ;
   wire [1:0]a_source  ;
   wire [1:0]b_source  ;
   wire [2:0]c_source  ;
   wire [1:0]pc_source      ;
   wire [3:0]mem_source     ;
   wire pause_mult  ;
   wire pause_ctrl    ;
   wire pause_pipeline;
   wire pause_any;
   wire pause_non_ctrl;
   wire pause_bank;
   wire nullify_op;
   wire intr_enable   ;
   reg intr_signal;
   wire exception_sig ;
   reg [3:0] reset_reg;
   wire reset         ;
   
    wire [31:0]c_mult2  ;
    wire pause_mult2  ;
/*
  wire [31:0]opcode2 ;
  wire pause_ctrl2    ;
  wire [31:0]c_memory2     ;
  wire [31:0]c_mult2       ;
  wire [31:2]address_next2;
  wire [3:0]byte_we_next2;
  wire [31:2]address2;
  wire [3:0] byte_we2;
  wire[31:0] data_w2;
  */
	assign pause_any = ((mem_pause | pause_ctrl) | (pause_mult | pause_pipeline));
	assign pause_non_ctrl = ((mem_pause | pause_mult) | pause_pipeline);
	assign pause_bank = ((mem_pause | pause_ctrl | pause_mult) & (~pause_pipeline));
	assign nullify_op = ((pc_source == FROM_LBRANCH && take_branch == 1'b0) || intr_signal == 1'b1 || exception_sig == 1'b1)? 1'b1 : 1'b0;
	assign c_bus = c_alu | c_shift | c_mult;
	assign reset = (reset_in == 1'b1 || reset_reg != 4'b1111)? 1'b1 : 1'b0;

	//synchronize reset and interrupt pins
	always @(posedge clk)
	begin
		if (reset_in == 1'b1)
		begin
			reset_reg = 4'b0000;
			intr_signal = 1'b0;
		end
		else
		begin
			if (reset_reg != 4'b1111)
				reset_reg = reset_reg + 1;
			
			//don't try to interrupt a multi-cycle instruction
			if (pause_any == 1'b0)
				if (intr_in == 1'b1 && intr_enable == 1'b1 && pc_source == FROM_INC4)  //the epc will contain pc+4
					intr_signal = 1'b1;
				else
					intr_signal = 1'b0;
		end
	end
   
	pc_next u1_pc_next(
        .clk        (clk),
        .reset_in   (reset),
        .take_branch(take_branch),
        .pause_in   (pause_any),
        .pc_new     (c_bus[31:2]),
        .opcode25_0 (opcode[25:0]),
        .pc_source  (pc_source),
        .pc_future  (pc_future),
        .pc_current (pc_current),
        .pc_plus4   (pc_plus4)
		);

	mem_ctrl u2_mem_ctrl 
		(
        .clk          (clk),
        .reset_in     (reset),
        .pause_in     (pause_non_ctrl),
        .nullify_op   (nullify_op),
        .address_pc   (pc_future),
        .opcode_out   (opcode),
        
        .address_in   (c_bus),
        .mem_source   (mem_source),
        .data_write   (reg_target),
        .data_read    (c_memory),
        .pause_out    (pause_ctrl),
        
        .address_next (address_next),
        .byte_we_next (byte_we_next),
             
        .address      (address),
        .byte_we      (byte_we),
        .data_w       (data_w),
        .data_r       (data_r)
		);
 /*   mem_ctrl2 u2_mem_ctrl2 
		(
        .clk          (clk),
        .reset_in     (reset),
        .pause_in     (pause_non_ctrl),
        .nullify_op   (nullify_op),
        .address_pc   (pc_future),
        .opcode_out   (opcode2),
        
        .address_in   (c_bus),
        .mem_source   (mem_source),
        .data_write   (reg_target),
        .data_read    (c_memory2),
        .pause_out    (pause_ctrl2),
        
        .address_next (address_next2),
        .byte_we_next (byte_we_next2),
             
        .address      (address2),
        .byte_we      (byte_we2),
        .data_w       (data_w2),
        .data_r       (data_r)
		);*/
	control u3_control(
        .opcode       	(opcode),
        .intr_signal  	(intr_signal),
        .rs_index     	(rs_index),
        .rt_index     	(rt_index),
        .rd_index     	(rd_index),
        .imm_out      	(imm),
        .alu_func     	(alu_func),
        .shift_func   	(shift_func),
        .mult_func    	(mult_func),
        .branch_func  	(branch_func),
        .a_source_out 	(a_source),
        .b_source_out 	(b_source),
        .c_source_out 	(c_source),
        .pc_source_out	(pc_source),
        .mem_source_out (mem_source),
        .exception_out 	( exception_sig));

	reg_bank u4_reg_bank	(
        .clk            (clk),
        .reset_in       (reset),
        .pause          (pause_bank),
        .rs_index       (rs_index),
        .rt_index       (rt_index),
        .rd_index       (rd_indexD),
        .reg_source_out (reg_source),
        .reg_target_out (reg_target),
        .reg_dest_new   (reg_destD),
        .intr_enable    (intr_enable));

	bus_mux u5_bus_mux(
        .imm_in       (imm),
        .reg_source   (reg_source),
        .a_mux        (a_source),
        .a_out        (a_bus),
                    
        .reg_target   (reg_target),
        .b_mux        (b_source),
        .b_out        (b_bus),
                      
        .c_bus        (c_bus),
        .c_memory     (c_memory),
        .c_pc         (pc_current),
        .c_pc_plus4   (pc_plus4),
        .c_mux        (c_source),
        .reg_dest_out (reg_dest),
                      
        .branch_func  (branch_func),
        .take_branch  (take_branch));

	alu u6_alu (
        .a_in         (a_busD),
        .b_in         (b_busD),
        .alu_function (alu_funcD),
        .c_alu        (c_alu));

	shifter u7_shifter(
        .value        (b_busD),
        .shift_amount (a_busD[4:0]),
        .shift_func   (shift_funcD),
        .c_shift      (c_shift));

  mult u8_mult(
        .clk       (clk),
        .reset_in  (reset),
        .a         (a_busD),
        .b         (b_busD),
        .mult_func (mult_funcD),
        .c_mult    (c_mult),
        .pause_out (pause_mult));

if  (pipeline_stages <= 2)
begin
    assign a_busD = a_bus;
    assign b_busD = b_bus;
    assign alu_funcD = alu_func;
    assign shift_funcD = shift_func;
    assign mult_funcD = mult_func;
    assign rd_indexD = rd_index;
    assign reg_destD = reg_dest;
    assign pause_pipeline = 1'b0;
end
   //pipeline3: if pipeline_stages > 2 generate
      //--When operating in three stage pipeline mode, the following signals
      //--are delayed by one clock cycle:  a_bus, b_bus, alu/shift/mult_func,
      //--c_source, and rd_index.
else
begin
	pipeline u9_pipeline(
        .clk            (clk),
        .reset          (reset),
        .a_bus          (a_bus),
        .a_busD         (a_busD),
        .b_bus          (b_bus),
        .b_busD         (b_busD),
        .alu_func       (alu_func),
        .alu_funcD      (alu_funcD),
        .shift_func     (shift_func),
        .shift_funcD    (shift_funcD),
        .mult_func      (mult_func),
        .mult_funcD     (mult_funcD),
        .reg_dest       (reg_dest),
        .reg_destD      (reg_destD),
        .rd_index       (rd_index),
        .rd_indexD      (rd_indexD),
                       
        .rs_index       (rs_index),
        .rt_index       (rt_index),
        .pc_source      (pc_source),
        .mem_source     (mem_source),
        .a_source       (a_source),
        .b_source       (b_source),
        .c_source       (c_source),
        .c_bus          (c_bus),
        .pause_any      (pause_any),
        .pause_pipeline (pause_pipeline));

end // --pipeline3

endmodule
