module plasma (clk , reset, uart_write, uart_read, address, byte_we, data_write, data_read, mem_pause_in, no_ddr_start, no_ddr_stop, gpio0_out, gpioA_in);
		
input clk   ;
input reset ;

output  uart_write  ;
input   uart_read   ;

output  [31:2]     address    ;
output  [3:0]     byte_we     ;
output  [31:0]     data_write ;
input   [31:0]    data_read   ;
input   mem_pause_in ;
output  no_ddr_start ;
output  no_ddr_stop  ;
        
output  [31:0]     gpio0_out;
input   [31:0]    gpioA_in  ;

   //-----------------------------------------
   
   parameter  ZERO          = 32'b00000000000000000000000000000000;
   parameter  ONES          = 32'b11111111111111111111111111111111;
   
   parameter  HIGH_Z        = 32'bZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ;

   parameter  ALU_NOTHING   = 4'b0000;
   parameter  ALU_ADD       = 4'b0001;
   parameter  ALU_SUBTRACT  = 4'b0010;
   parameter  ALU_LESS_THAN = 4'b0011;
   parameter  ALU_LESS_THAN_SIGNED = 4'b0100;
   parameter  ALU_OR        = 4'b0101;
   parameter  ALU_AND       = 4'b0110;
   parameter  ALU_XOR       = 4'b0111;
   parameter  ALU_NOR       = 4'b1000;

   parameter  SHIFT_NOTHING        = 2'b00;
   parameter  SHIFT_LEFT_UNSIGNED  = 2'b01;
   parameter  SHIFT_RIGHT_SIGNED   = 2'b11;
   parameter  SHIFT_RIGHT_UNSIGNED = 2'b10;

   parameter  MULT_NOTHING       = 4'b0000;
   parameter  MULT_READ_LO       = 4'b0001;
   parameter  MULT_READ_HI       = 4'b0010;
   parameter  MULT_WRITE_LO      = 4'b0011;
   parameter  MULT_WRITE_HI      = 4'b0100;
   parameter  MULT_MULT          = 4'b0101;
   parameter  MULT_SIGNED_MULT   = 4'b0110;
   parameter  MULT_DIVIDE        = 4'b0111;
   parameter  MULT_SIGNED_DIVIDE = 4'b1000;

   parameter  A_FROM_REG_SOURCE = 2'b00;
   parameter  A_FROM_IMM10_6    = 2'b01;
   parameter  A_FROM_PC         = 2'b10;

   parameter  B_FROM_REG_TARGET = 2'b00;
   parameter  B_FROM_IMM        = 2'b01;
   parameter  B_FROM_SIGNED_IMM = 2'b10;
   parameter  B_FROM_IMMX4      = 2'b11;

   parameter  C_FROM_NULL       = 3'b000;
   parameter  C_FROM_ALU        = 3'b001;
   parameter  C_FROM_SHIFT      = 3'b001;
   parameter  C_FROM_MULT       = 3'b001; 
   parameter  C_FROM_MEMORY     = 3'b010;
   parameter  C_FROM_PC         = 3'b011;
   parameter  C_FROM_PC_PLUS4   = 3'b100;
   parameter  C_FROM_IMM_SHIFT16= 3'b101;
   parameter  C_FROM_REG_SOURCEN= 3'b110;

   parameter  FROM_INC4       = 2'b00;
   parameter  FROM_OPCODE25_0 = 2'b01;
   parameter  FROM_BRANCH     = 2'b10;
   parameter  FROM_LBRANCH    = 2'b11;

   parameter  BRANCH_LTZ = 3'b000;
   parameter  BRANCH_LEZ = 3'b001;
   parameter  BRANCH_EQ  = 3'b010;
   parameter  BRANCH_NE  = 3'b011;
   parameter  BRANCH_GEZ = 3'b100;
   parameter  BRANCH_GTZ = 3'b101;
   parameter  BRANCH_YES = 3'b110;
   parameter  BRANCH_NO  = 3'b111;


   parameter  MEM_FETCH   = 4'b0000;
   parameter  MEM_READ32  = 4'b0100;
   parameter  MEM_WRITE32 = 4'b0101;
   parameter  MEM_READ16  = 4'b1000;
   parameter  MEM_READ16S = 4'b1010;
   parameter  MEM_WRITE16 = 4'b1001;
   parameter  MEM_READ8   = 4'b1100;
   parameter  MEM_READ8S  = 4'b1110;
   parameter  MEM_WRITE8  = 4'b1101;  
   

   //----------------------------------------
   


//architecture logic of plasma is
   wire [31:2] address_next  ;
   wire [3:0] byte_we_next   ;
   wire [31:0] cpu_address   ;
   wire [3:0] cpu_byte_we    ;
   wire [31:0] cpu_data_w    ;
   reg [31:0] cpu_data_r     ;
   wire cpu_pause        ;

   wire [7:0] data_read_uart   ;
   wire write_enable     ;
   wire eth_pause_in     ;
   wire eth_pause        ;
   wire mem_busy         ;

   wire enable_misc      ;
   wire enable_uart      ;
   wire enable_uart_read ;
   wire enable_uart_write;
   wire enable_eth       ;

   reg [31:0] gpio0_reg  ;
   wire uart_write_busy  ;
   wire uart_data_avail  ;
   reg [7:0] irq_mask_reg;
   wire [7:0] irq_status ;
   wire irq              ;
   wire irq_eth_rec      ;
   wire irq_eth_send     ;
   reg [31:0] counter_reg;

   reg ram_enable        ;
   reg [3:0] ram_byte_we ;
   reg [31:2] ram_address;
   reg [31:0] ram_data_w ;
   wire [31:0] ram_data_r;

   wire cache_access     ;
   wire cache_checking   ;
   wire cache_miss       ;
   wire cache_hit        ;

//begin  //architecture
   assign write_enable = (cpu_byte_we != 4'b0000)? 1'b1:1'b0;
   
   assign mem_busy = eth_pause | mem_pause_in;
   
   assign cache_hit = cache_checking & ~cache_miss;
   
   assign cpu_pause = (uart_write_busy & enable_uart & write_enable) |  //UART busy
      cache_miss |                                                 //Cache wait
      (cpu_address[28] & ~cache_hit & mem_busy);                //DDR || flash
	  
	  
   assign irq_status = {gpioA_in[31] , ~gpioA_in[31] ,
                 irq_eth_send , irq_eth_rec , 
                 counter_reg[18] , ~counter_reg[18] ,
                 ~uart_write_busy , uart_data_avail};
				 
   assign irq = ((irq_status & irq_mask_reg) != ZERO[7:0])? 1'b1:1'b0;
   
   assign gpio0_out[31:29] = gpio0_reg[31:29];
   
   assign gpio0_out[23:0] = gpio0_reg[23:0];

   assign enable_misc = (cpu_address[30:28] == 3'b010)? 1'b1:1'b0;
   
   assign enable_uart = (enable_misc == 1'b1 && cpu_address[7:4] == 4'b0000)? 1'b1:1'b0;
   assign enable_uart_read = enable_uart & ~write_enable;
   assign enable_uart_write = enable_uart & write_enable;
   assign enable_eth = (enable_misc == 1'b1 && cpu_address[7:4] == 4'b0111)? 1'b1:1'b0;
   assign cpu_address[1:0] = 2'b00;

    mlite_cpu u1_cpu (
          clk,
          reset,
          irq,
 
          address_next,             //bef||e rising_edge(clk)
          byte_we_next,

          cpu_address[31:2], //after rising_edge(clk)
          cpu_byte_we,
          cpu_data_w,
          cpu_data_r,
          cpu_pause);

   cache u_cache (
         .clk            (clk),
         .reset          (reset),
         .address_next   (address_next),
         .byte_we_next   (byte_we_next),
         .cpu_address    (cpu_address[31:2]),
         .mem_busy       (mem_busy),
         
         .cache_access   (cache_access),    //--access 4KB cache
         .cache_checking (cache_checking),  //--checking if cache hit
         .cache_miss     (cache_miss));     //--cache miss	
	
	
   assign no_ddr_start = ~eth_pause & cache_checking;
   assign no_ddr_stop = ~eth_pause & cache_miss;
   assign eth_pause_in = mem_pause_in | (~eth_pause & cache_miss & ~cache_checking);

   
	always @ ( cpu_address, enable_misc, ram_data_r, data_read, data_read_uart, cpu_pause, irq_mask_reg, irq_status, gpio0_reg, write_enable, cache_checking, gpioA_in, counter_reg, cpu_data_w)
	begin
		case (cpu_address[30:28])
			3'b000:         //internal RAM
				cpu_data_r <= ram_data_r;
			3'b001:         //external RAM
				if (cache_checking == 1'b1) 
				begin
				cpu_data_r <= ram_data_r; //cache
				end
				else
				begin
					cpu_data_r <= data_read; //DDR
				end
			3'b010:         //misc
				case (cpu_address[6:4])
					3'b000:      //uart
						cpu_data_r <= {ZERO[31:8] , data_read_uart};
					3'b001:      //irq_mask
						cpu_data_r <= {ZERO[31:8] , irq_mask_reg};
					3'b010:      //irq_status
						cpu_data_r <= {ZERO[31:8] , irq_status};
					3'b011:      //gpio0
						cpu_data_r <= gpio0_reg;
					3'b101:      //gpioA
						cpu_data_r <= gpioA_in;
					3'b110:      //counter
						cpu_data_r <= counter_reg;        
					default:
						cpu_data_r <= gpioA_in;
				endcase
			3'b011:         //flash
				cpu_data_r <= data_read;
			default:
				cpu_data_r <= ZERO;
		endcase
	end	  
 
	always@(posedge(clk))
	begin

		if (reset == 1'b1)  
		begin
			irq_mask_reg <= ZERO[7:0];
			gpio0_reg <= ZERO;
			counter_reg <= ZERO;
		end
		else
		begin
			if (cpu_pause == 1'b0)  begin
				if (enable_misc == 1'b1 && write_enable == 1'b1)  begin
					if (cpu_address[6:4] == 3'b001)  
					begin
						irq_mask_reg <= cpu_data_w[7:0];
					end
					else if (cpu_address[6:4] == 3'b011)
					begin
						gpio0_reg <= gpio0_reg | cpu_data_w;
					end
					else if (cpu_address[6:4] == 3'b100)
					begin
						gpio0_reg <= gpio0_reg & ~cpu_data_w;
					end
				end
			end
			//counter_reg <= bv_inc(counter_reg);
			counter_reg <= counter_reg + 1;
		end
	end
   

	always @(cache_access, cache_miss,  address_next, cpu_address,  byte_we_next, cpu_data_w, data_read)
	begin
		if (cache_access == 1'b1)
		begin    //Check if cache hit || write through
			ram_enable <= 1'b1;
			ram_byte_we <= byte_we_next;
			ram_address[31:2] <= {ZERO[31:16], 4'b0001 , address_next[11:2]};
			ram_data_w <= cpu_data_w;
		end
		else if (cache_miss == 1'b1)
		begin  //Update cache after cache miss
			ram_enable <= 1'b1;
			ram_byte_we <= 4'b1111;
			ram_address[31:2] <= {ZERO[31:16] , 4'b0001 , cpu_address[11:2]};
			ram_data_w <= data_read;
		end
		else
		begin
			if (address_next[30:28] == 3'b000)
			begin
				ram_enable <= 1'b1;
			end	
			else
			begin
				ram_enable <= 1'b0;
			end
			ram_byte_we <= byte_we_next;
			ram_address[31:2] <= address_next[31:2];
			ram_data_w <= cpu_data_w;
		end
	end

    ram u2_ram(
         clk,
		 reset,
         ram_enable,
         ram_byte_we,
         ram_address,
         ram_data_w,
         ram_data_r);

   uart u3_uart(
        clk,
        reset,
        enable_uart_read,
        enable_uart_write,
        cpu_data_w[7:0],
        data_read_uart,
        uart_read,
        uart_write,
        uart_write_busy,
        uart_data_avail);

   eth_dma u4_eth (
         .clk         (clk),
         .reset       (reset),
         .enable_eth  (gpio0_reg[24]),
         .select_eth  (enable_eth),
         .rec_isr     (irq_eth_rec),
         .send_isr    (irq_eth_send),
         
         .address     (address),//      --to DDR
         .byte_we     (byte_we),
         .data_write  (data_write),
         .data_read   (data_read),
         .pause_in    (eth_pause_in),
         
         .mem_address (cpu_address[31:2]),// --from CPU
         .mem_byte_we (cpu_byte_we),
         .data_w      (cpu_data_w),
         .pause_out   (eth_pause),
         
         .E_RX_CLK    (gpioA_in[20]),
         .E_RX_DV     (gpioA_in[19]),
         .E_RXD       (gpioA_in[18:15]),
         .E_TX_CLK    (gpioA_in[14]),
         .E_TX_EN     (gpio0_out[28]),
         .E_TXD       (gpio0_out[27:24]));

endmodule
