

module reg_bank
       (clk            ,
        reset_in       ,
        pause          ,
        rs_index       ,
        rt_index       ,
        rd_index       ,
        reg_source_out ,
        reg_target_out ,
        reg_dest_new   ,
        intr_enable    );

input		clk            ;//in  std_logic;
input        reset_in       ;//in  std_logic;
input        pause          ;//in  std_logic;
input [5:0]       rs_index       ;//in  std_logic_vector(5:0);
input [5:0]       rt_index       ;//in  std_logic_vector(5:0);
input [5:0]       rd_index       ;//in  std_logic_vector(5:0);
output reg [31:0]       reg_source_out ;//out std_logic_vector(31:0);
output reg [31:0]       reg_target_out ;//out std_logic_vector(31:0);
input  [31:0]     reg_dest_new   ;//in  std_logic_vector(31:0);
output reg        intr_enable    ;//out std_logic;
		
parameter  ZERO          = 32'b00000000000000000000000000000000;

reg intr_enable_reg ;//: std_logic;
//type ram_type is array(31:0) of std_logic_vector(31:0);
reg [31:0] tri_port_ram [31:0];

//controls access to dual-port memories
reg [4:0] addr_read1, addr_read2 ;//: std_logic_vector(4:0);
reg [4:0] addr_write             ;//: std_logic_vector(4:0);
reg [31:0] data_out1, data_out2   ;//: std_logic_vector(31:0);
reg write_enable           ;//: std_logic;

  
always @(rs_index, rt_index, rd_index, reg_dest_new, intr_enable_reg, data_out1, data_out2, reset_in, pause)
begin
	//setup for first dual-port memory
	if (rs_index == 6'b101110)// then  //reg_epc CP0 14
	begin
		addr_read1 <= 5'b00000;
	end
	else
	begin
		addr_read1 <= rs_index[4:0];
	end
	
	case (rs_index)
		6'b000000: reg_source_out <= ZERO;
		6'b101100: reg_source_out <= {ZERO[31:1] , intr_enable_reg};
		6'b111111: reg_source_out <= {ZERO[31:8] , 8'b00111100};
		default :reg_source_out <= data_out1;
	endcase

	//setup for second dual-port memory
	addr_read2 <= rt_index[4:0];
	case (rt_index)
		6'b000000: reg_target_out <= ZERO;
		default:    reg_target_out <= data_out2;
	endcase

	//setup write port for both dual-port memories
	if (rd_index != 6'b000000 && rd_index != 6'b101100 && pause == 1'b0)
	begin
		write_enable <= 1'b1;
	end
	else
	begin
		write_enable <= 1'b0;
	end
	
	if (rd_index == 6'b101110)
	begin
		addr_write <= 5'b00000;
	end
	else
	begin
		addr_write <= rd_index[4:0];
	end

	intr_enable <= intr_enable_reg;
end

always @(posedge(clk))
begin
	if (reset_in == 1'b1)
	begin
		intr_enable_reg <= 1'b0;
	end
	else
	begin
		if (rd_index == 6'b101110)
		begin
			intr_enable_reg <= 1'b0;      //disable interrupts
		end
		else if (rd_index == 6'b101100)
		begin
			intr_enable_reg <= reg_dest_new[0];
		end
	end
end

always @(posedge(clk))
begin
	if (write_enable == 1'b1)
	begin
		tri_port_ram[addr_write] <= reg_dest_new;
	end
end

always @(addr_read1, addr_read2,addr_write, reg_dest_new, write_enable)
//variable tri_port_ram : ram_type := (others => ZERO);
begin
	data_out1 <= tri_port_ram[addr_read1];
	data_out2 <= tri_port_ram[addr_read2];
end
   
endmodule
