/*
---------------------------------------------------------------------
-- TITLE: Shifter Unit
-- AUTHOR: Steve Rhoads (rhoadss@yahoo.com)
--         Matthias Gruenewald
-- DATE CREATED: 2/2/01
-- FILENAME: shifter.vhd
-- PROJECT: Plasma CPU core
-- COPYRIGHT: Software placed into the public domain by the author.
--    Software 'as is' without warranty.  Author liable for nothing.
-- DESCRIPTION:
--    Implements the 32-bit shifter unit.
---------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use work.mlite_pack.all;
*/

module shifter(value,
        shift_amount,
        shift_func,
        c_shift);
		
input [31:0] value;
input [4:0]  shift_amount;
input   [1:0]  shift_func;
output [31:0] c_shift;

parameter  SHIFT_NOTHING        =2'b 00;
parameter  SHIFT_LEFT_UNSIGNED  =2'b 01;
parameter  SHIFT_RIGHT_SIGNED   =2'b 11;
parameter  SHIFT_RIGHT_UNSIGNED =2'b 10;
parameter  ZERO = 32'b 00000000000000000000000000000000;  

wire [31:0]  shift1L, shift2L, shift4L, shift8L, shift16L, shift1R, shift2R, shift4R, shift8R, shift16R;
wire [31:16]  fills;

   assign fills = (shift_func == SHIFT_RIGHT_SIGNED && value[31] == 1'b 1)? 16'b 1111111111111111: 16'b 0000000000000000;
   
   assign shift1L  = (shift_amount[0] == 1'b 1)? {value[30:0] , 1'b 0}:value;
  
  assign shift2L  = (shift_amount[1] == 1'b 1)? {shift1L[29:0], 2'b 00}:shift1L;
   assign shift4L  = (shift_amount[2] == 1'b 1)? {shift2L[27:0],4'b 0000} : shift2L;
   assign shift8L  = (shift_amount[3] == 1'b 1)? {shift4L[23:0],8'b 00000000} : shift4L;
   assign shift16L = (shift_amount[4] == 1'b 1)? {shift8L[15:0],ZERO[15:0]} : shift8L;

   assign shift1R  = (shift_amount[0] == 1'b 1)? {fills[31],value[31:1]} : value;
   assign shift2R  = (shift_amount[1] == 1'b 1)? {fills[31:30],shift1R[31:2]} : shift1R;
   assign shift4R  = (shift_amount[2] == 1'b 1)? {fills[31:28],shift2R[31:4]} : shift2R;
   assign shift8R  = (shift_amount[3] == 1'b 1)? {fills[31:24],shift4R[31:8]}  : shift4R;
   assign shift16R = (shift_amount[4] == 1'b 1)? {fills[31:16],shift8R[31:16]} : shift8R;

   assign c_shift = (shift_func == SHIFT_LEFT_UNSIGNED)? shift16L : (shift_func == SHIFT_RIGHT_UNSIGNED || shift_func == SHIFT_RIGHT_SIGNED)? shift16R:ZERO;

endmodule

