module mult ( clk, reset_in, a, b, mult_func, c_mult, pause_out);
input clk;
input reset_in;
input [31:0] a;
input [31:0] b;
input [3:0] mult_func;
output [31:0] c_mult;
output pause_out;

parameter MODE_MULT = 1'b1;
parameter MODE_DIV  = 1'b0;
parameter MULT_NOTHING       = 4'b0000;
parameter MULT_READ_LO       = 4'b0001;
parameter MULT_READ_HI       = 4'b0010;
parameter MULT_WRITE_LO      = 4'b0011;
parameter MULT_WRITE_HI      = 4'b0100;
parameter MULT_MULT          = 4'b0101;
parameter MULT_SIGNED_MULT   = 4'b0110;
parameter MULT_DIVIDE        = 4'b0111;
parameter MULT_SIGNED_DIVIDE = 4'b1000;
parameter ZERO          		= 32'b00000000000000000000000000000000;

reg mode_reg;
reg negate_reg;
reg sign2_reg;
reg sign_reg;
reg [5:0] count_reg;
reg [31:0] aa_reg;
reg [31:0] bb_reg;
reg [31:0] upper_reg;
reg [31:0] lower_reg;

wire [31:0] a_neg;
wire [31:0] b_neg;
wire [32:0] sum;
reg [2:0] count;

 
   // Result
	assign c_mult = (mult_func == MULT_READ_LO && negate_reg == 1'b0)? lower_reg  : (mult_func == MULT_READ_LO && negate_reg == 1'b1)? 
             ((~lower_reg)+1): (mult_func == MULT_READ_HI)? upper_reg : ZERO;
             
	assign pause_out = ((count_reg != 6'b000000) && (mult_func == MULT_READ_LO || mult_func == MULT_READ_HI))? 1'b1 : 1'b0;

	//-- ABS and remainder signals
	assign a_neg = (~a)+1;
	assign b_neg = (~b)+1 ;
	assign sum = (mode_reg == 1'b1)? upper_reg+aa_reg : upper_reg-aa_reg;
    
   //multiplication/division unit
always @(posedge clk) 
begin
    count <= 3'b001;
    if (reset_in == 1'b1)
	begin
        mode_reg 	<= 1'b0;
        negate_reg 	<= 1'b0;
        sign_reg 	<= 1'b0;
        sign2_reg 	<= 1'b0;
        count_reg 	<= 6'b000000;
        aa_reg <= ZERO;
        bb_reg <= ZERO;
        upper_reg <= ZERO;
        lower_reg <= ZERO;
	end
    else
	begin
        case (mult_func)
            MULT_WRITE_LO :
			begin
               lower_reg <= a;
               negate_reg <= 1'b0;
			end
            MULT_WRITE_HI :
			begin
               upper_reg <= a;
               negate_reg <= 1'b0;
            end
			MULT_MULT :
			begin
               mode_reg <= MODE_MULT;
               aa_reg <= a;
               bb_reg <= b;
               upper_reg <= ZERO;
               count_reg <= 6'b100000;
               negate_reg 	<= 1'b0;
               sign_reg 	<= 1'b0;
               sign2_reg 	<= 1'b0;
			end
            MULT_SIGNED_MULT :
			begin
				mode_reg <= MODE_MULT;
				if (b[31] == 1'b0)
				begin
					aa_reg 	<= a;
					bb_reg 	<= b;
				end
				else
				begin
					aa_reg <= a_neg;
					bb_reg <= b_neg;
					
				end
				sign_reg  <= a[31] ^ b[31];
				sign2_reg <= 1'b0;
				upper_reg <= ZERO;
				count_reg <= 6'b100000;
				negate_reg <= 1'b0;
			end
            MULT_DIVIDE :
			begin
               mode_reg <= MODE_DIV;
               aa_reg <= {b[0] , ZERO[30:0]};
               bb_reg <= b;
               upper_reg <= a;
               count_reg <= 6'b100000;
               negate_reg <= 1'b0;
			end
            MULT_SIGNED_DIVIDE :
			begin
				mode_reg <= MODE_DIV;
				if (b[31] == 1'b0)
				begin
					aa_reg[31] <= b[0];
					bb_reg <= b;
				end
				else
				begin
					aa_reg[31] <= b_neg[0];
					bb_reg <= b_neg;
				end
				if ( a[31] == 1'b0)
					upper_reg <= a;
				else
					upper_reg <= a_neg;
				aa_reg[30:0] <= ZERO[30:0];
				count_reg <= 6'b100000;
				negate_reg <= a[31] ^ b[31];
			end
            default:
			begin
				if (count_reg != 6'b000000)
				begin
					if ( mode_reg == MODE_MULT )
                     // Multiplication
						if ( bb_reg[0] == 1'b1)
						begin
							upper_reg <= {(sign_reg ^ sum[32]) , sum[31:1]};
							lower_reg <= {sum[0] , lower_reg[31:1]};
							sign2_reg <= sign2_reg | sign_reg;
							sign_reg <= 1'b0;
							bb_reg <= {1'b0, bb_reg[31:1]};
						end
						//-- The following six lines are optional for speedup
						//else if ( (bb_reg[3:0] == 4'b0000) && (sign2_reg == 1'b0) && (count_reg[5:2] != 4'b0000) )
						//begin
						//	upper_reg = {4'b0000 , upper_reg[31:4]};
						//	lower_reg =  {upper_reg[3:0] , lower_reg[31:4]};
						//	count = 3'b100;
						//	bb_reg = {4'b0000 , bb_reg[31:4]};
						//end
						else
						begin
							
							upper_reg <= {sign2_reg, upper_reg[31:1]};	
							lower_reg <= {upper_reg[0], lower_reg[31:1]};
							bb_reg <= {1'b0 , bb_reg[31:1]};
						end
					else   
                     //-- Division
					begin
					
						if ( (sum[32] == 1'b0) && (aa_reg != ZERO) && (bb_reg[31:1] == ZERO[31:1]) )
						begin
							upper_reg <= sum[31:0];
							lower_reg[0] <= 1'b1;
						end
						else
						begin
							lower_reg[0] <= 1'b0;
						end
						aa_reg <= {bb_reg[1] , aa_reg[31:1]};
						lower_reg[31:1] <= lower_reg[30:0];
						bb_reg <= {1'b0 , bb_reg[31:1]};
					end
					count_reg <= count_reg - count;
				end 
			end
        endcase     
	end
end
    
endmodule
