module pc_next( clk, reset_in, pc_new, take_branch, pause_in, opcode25_0, pc_source, pc_future, pc_current, pc_plus4);
input clk;
input reset_in;
input [31:2] pc_new;
input take_branch;
input pause_in;
input [25:0] opcode25_0;
input [1:0] pc_source;
output [31:2] pc_future;
output [31:2] pc_current;
output [31:2] pc_plus4;



parameter FROM_INC4       = 2'b00;
parameter FROM_OPCODE25_0 = 2'b01;
parameter FROM_BRANCH     = 2'b10;
parameter FROM_LBRANCH    = 2'b11;
parameter ZERO          		= 32'b00000000000000000000000000000000;

reg [31:2] pc_reg;
reg [31:2] pc_inc;
reg [31:2] pc_next;

 
always @(reset_in, pc_new, take_branch, pause_in, 
                 opcode25_0, pc_source, pc_reg)
  
begin
   pc_inc = pc_reg+1;  //--pc_reg+1

   case (pc_source)
	FROM_INC4:
		pc_next = pc_inc;
	FROM_OPCODE25_0 :
		pc_next = {pc_reg[31:28],opcode25_0};
	FROM_BRANCH:
		if (take_branch == 1'b1)
			pc_next = pc_new;
		else
			pc_next = pc_inc;
      
	FROM_LBRANCH:
		if (take_branch == 1'b1)
			pc_next = pc_new;
		else
			pc_next = pc_inc;
      
	default:
		pc_next = pc_inc;
   endcase

	if (pause_in == 1'b1)
		pc_next = pc_reg;
   
	if ( reset_in == 1'b1) 
		pc_next = pc_reg;
  

   
end

assign pc_future = pc_next;
assign pc_current = pc_reg;
assign pc_plus4 = pc_inc;

always @(posedge clk)  //, posedge reset_in)
begin
	if ( reset_in == 1'b1) 
		pc_reg = ZERO[31:2];
	else
      pc_reg = pc_next;
	
end

endmodule