module uart(clk, reset, enable_read, enable_write, data_in, data_out, uart_read, uart_write, busy_write, data_avail);
input clk;
input reset;
input enable_read;
input enable_write;
input [7:0] data_in;
output reg [7:0] data_out;
input uart_read;
output reg uart_write;
output reg busy_write;
output reg data_avail;
      
reg [9:0] delay_write_reg;
reg [3:0] bits_write_reg;
reg [8:0] data_write_reg;
reg [9:0] delay_read_reg;
reg [3:0] bits_read_reg;
reg [7:0] data_read_reg;
reg [17:0] data_save_reg;
reg busy_write_sig;
reg [6:0] read_value_reg;
reg uart_read2;


parameter ZERO  = 32'b00000000000000000000000000000000;	
parameter log_file = "output.txt";
parameter COUNT_VALUE =	10'b0110110010;  

integer testFile;	
	

//     "0100011110";  --33MHz/2/57600Hz = 0x11e
//      "1101100100";  --50MHz/57600Hz = 0x364
//      "0110110010";  --25MHz/57600Hz = 0x1b2 -- Plasma IF uses div2
//      "0011011001";  --12.5MHz/57600Hz = 0xd9
//      "0000000100";  --for debug (shorten read_value_reg)	  
always @(posedge clk)
begin
	if (reset == 1'b1)
	begin
		data_write_reg  = {ZERO[8:1] , 1'b1};
		bits_write_reg  = 4'b0000;
		delay_write_reg = ZERO[9:0];
		read_value_reg  = 7'b1111111;   //ONES[6 downto 0];
		data_read_reg   = ZERO[7:0];
		bits_read_reg   = 4'b0000;
		delay_read_reg  = ZERO[9:0];
		data_save_reg   = ZERO[17:0];
	end
	else
	begin
      //Write UART
		if (bits_write_reg == 4'b0000)               // --nothing left to write?
		begin
			if (enable_write == 1'b1)
			begin
				delay_write_reg = ZERO[9:0];    //delay before next bit
				bits_write_reg = 4'b1010;               //number of bits to write
				data_write_reg = {data_in, 1'b0};        //remember data & start bit
			end
		end
		else
		begin
			if (delay_write_reg != COUNT_VALUE)
			begin
				delay_write_reg = delay_write_reg + 1; //--delay before next bit
			end
			else
			begin
				delay_write_reg = ZERO[9:0];    //--reset delay
				bits_write_reg = bits_write_reg - 1;   //--bits left to write
				data_write_reg = {1'b1, data_write_reg[8:1]};
			end
		end

	//--Average uart_read signal
		if (uart_read == 1'b1)
		begin
			if (read_value_reg != 7'b1111111)
			begin
				read_value_reg = read_value_reg + 1;
			end
			else
			if (read_value_reg != ZERO[6:0])
			begin
				read_value_reg = read_value_reg - 1;
			end
		end
    //  --Read UART
		if (delay_read_reg == ZERO[9:0])    // --done delay for read?
		begin
			if (bits_read_reg == 4'b0000)  //             --nothing left to read?
			begin
				if (uart_read2 == 1'b0)           //    --wait for start bit
				begin
					delay_read_reg = {1'b0, COUNT_VALUE[9:1]}; // --half period
					bits_read_reg = 4'b1001;          //   --bits left to read
				end
			end
			else
			begin
				delay_read_reg = COUNT_VALUE;         // --initialize delay
				bits_read_reg = bits_read_reg - 1;   //  --bits left to read
				data_read_reg = {uart_read2, data_read_reg[7:1]};
			end
		end
		else
		begin
			delay_read_reg = delay_read_reg - 1;      //--delay
		end

     // --Control character buffer
		if (bits_read_reg == 4'b0000 && delay_read_reg == COUNT_VALUE )
		begin
			if (data_save_reg[8] == 1'b0 || (enable_read == 1'b1 && data_save_reg[17] == 1'b0)) // --Empty buffer
			begin
				data_save_reg[8:0] = {1'b1 , data_read_reg};
			end
			else
			begin
				//--Second character in buffer
				if (enable_read == 1'b1)
				begin
					data_save_reg[8:0] = data_save_reg[17:9];
				end
				data_save_reg[17:9] = {1'b1, data_read_reg};
			end
		end
		else if (enable_read == 1'b1)
		begin
			        //         --data_available
			data_save_reg[8:0] = data_save_reg[17:9];
			data_save_reg[17] = 1'b0; 
		end
	end
end


				   
always @(enable_read, enable_write, data_in, data_write_reg, bits_write_reg, delay_write_reg, 
        data_read_reg, bits_read_reg, delay_read_reg, data_save_reg, read_value_reg, uart_read2, uart_read)
begin
   uart_read2 = read_value_reg[6];
   uart_write = data_write_reg[0];
   
   if (bits_write_reg != 4'b0000 && log_file == "UNUSED")   
// Comment out the following line for full UART simulation (much slower)
		busy_write_sig = 1'b1;
	else
		busy_write_sig = 1'b0;
	
	busy_write = busy_write_sig;
	data_avail = data_save_reg[8];
	data_out = data_save_reg[7:0];
end 


//synthesis_off
if (log_file != "UNUSED" )
begin
	initial
	begin
		testFile = $fopen(log_file, "w");
	end

   always @( posedge clk)
	begin
			if (busy_write_sig == 1'b0)
				if (enable_write == 1'b1)
						$fwrite(testFile, "%c", {1'b0,data_in[6:0]});
	end	
end	
//--The following line may have to be commented out for synthesis

endmodule
